<%-- $Id$
	------------------------------------------------------------------
	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
--%>
<%@ page pageEncoding="UTF-8" session="false" contentType="application/json"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.SyncFilter"
	import="net.sf.basedb.core.SyncFilter.SourceItemTransform"
	import="net.sf.basedb.core.PropertyFilter"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.error.ThrowableUtil"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.util.formatter.PropertyFilterFormatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="org.json.simple.JSONObject"
	import="java.util.Date"
	import="java.util.List"
%>
<%!
	String formatFilter(DbControl dc, SyncFilter syncFilter)
	{
		StringBuilder sb = new StringBuilder();
		Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(dc.getSessionControl());
		Formatter<PropertyFilter> filterFormatter = new PropertyFilterFormatter(dc, dateFormatter, true);
		for (int filterRow = 0; filterRow < syncFilter.getFilterRows(); filterRow++)
		{
			sb.append(filterRow > 0 ? "\n<operator>OR</operator> " : "");
			List<String> filters = SyncFilter.getFormattedAndSortedFilters(syncFilter.getPropertyFilters(filterRow), filterFormatter);
			sb.append(Values.getString(filters, "\n   <operator>AND</operator> ", true));
		}
		return sb.toString();
	}
%>
<%
response.setHeader("Cache-Control", "no-cache, max-age=0");
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";
out.clear();
JSONObject json = new JSONObject();
json.put("status", "ok");
DbControl dc = null;
try
{
	if ("GetCurrentFilter".equals(cmd))
	{
		dc = sc.newDbControl();
		
		// Get the current sync filter item we are editing
		ItemContext cc = sc.getCurrentContext(Item.SYNCFILTER);
		SyncFilter syncFilter = (SyncFilter)cc.getObject("item");
		
		// Get the requested source item type in the edit dialog
		Item sourceItemType = Item.valueOf(request.getParameter("sourceItemType"));
		SourceItemTransform sourceItemTransform = SourceItemTransform.valueOf(request.getParameter("sourceItemTransform"));
		
		if (sourceItemTransform != syncFilter.getSourceItemTransformation())
		{
			syncFilter.setSyncContext(null, sourceItemTransform);
		}

		// If they match we return the formatted filter, otherwise not
		if (sourceItemType == syncFilter.getSourceItemType())
		{
			json.put("filter", HTML.encodeTags(formatFilter(dc, syncFilter), "value|name|operator|function"));
		}
	}
	else if ("CopySyncFilterToItemContext".equals(cmd))
	{
		dc = sc.newDbControl();
		int syncFilterId = Values.getInt(request.getParameter("syncfilter_id"));
		String srcType = Values.getStringOrNull(request.getParameter("sourceItemType"));
		
		// Get the current sync filter item we are editing
		SyncFilter syncFilter;
		Item sourceItemType;
		if (syncFilterId == 0)
		{
			ItemContext cc = sc.getCurrentContext(Item.SYNCFILTER);
			syncFilter = (SyncFilter)cc.getObject("item");
		}
		else
		{
			syncFilter = SyncFilter.getById(dc, syncFilterId);
		}
		
		if (srcType == null)
		{
			sourceItemType = syncFilter.getSourceItemType();
		}
		else
		{
			// Get the requested source item type in the edit dialog
			sourceItemType = Item.valueOf(srcType);
		}
		
		// If they match we copy the filter to the requested item context
		if (sourceItemType == syncFilter.getSourceItemType())
		{
			syncFilter.copyToContext(sc.getCurrentContext(sourceItemType));
		}
	}
	else if ("CopyItemContextToSyncFilter".equals(cmd))
	{
		dc = sc.newDbControl();
		int syncFilterId = Values.getInt(request.getParameter("syncfilter_id"));

		SyncFilter syncFilter;
		Item sourceItemType;
		SourceItemTransform transform;
		if (syncFilterId == 0)
		{
			// Get the current sync filter item we are editing
			ItemContext cc = sc.getCurrentContext(Item.SYNCFILTER);
			syncFilter = (SyncFilter)cc.getObject("item");

			// Get the requested source item type in the edit dialog
			sourceItemType = Item.valueOf(request.getParameter("sourceItemType"));
			transform = SourceItemTransform.valueOf(request.getParameter("sourceItemTransform"));
		}
		else
		{
			syncFilter = SyncFilter.getById(dc, syncFilterId);
			sourceItemType = syncFilter.getSourceItemType();
			transform = syncFilter.getSourceItemTransformation();
		}
		
		// Copy back to the current filter
		syncFilter.setSyncContext(sc.getCurrentContext(sourceItemType), transform);
		
		json.put("filter", HTML.encodeTags(formatFilter(dc, syncFilter), "value|name|operator|function"));
		dc.commit();
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
catch (Throwable t)
{
	t.printStackTrace();
	json.clear();
	json.put("status", "error");
	json.put("message", t.getMessage());
	json.put("stacktrace", ThrowableUtil.stackTraceToString(t));
}
finally
{
	json.writeJSONString(out);
	out.flush();
	if (dc != null) dc.close();
}
%>